@@ -1,4 +1,4 @@
-from .render_out_mock import RenderOutMock
 from .robots import RobotJiminyEnv, RobotJiminyGoalEnv
+from .render_out_mock import RenderOutMock
 from .play import loop_interactive
-from .flatten_observation import FlattenObservation
+from .wrappers import FlattenObservation, ObservationActionNormalization, flatten_observation
